<?php

/**
 * Copyright (c) 2019 Nexi Payments S.p.A.
 *
 * @author      iPlusService S.r.l.
 * @category    Payment Module
 * @package     Nexi XPay
 * @version     6.0.0
 * @copyright   Copyright (c) 2019 Nexi Payments S.p.A. (https://ecommerce.nexi.it)
 * @license     GNU General Public License v3.0
 */

namespace Nexi;

class WC_Admin_Page
{

    public static function init()
    {
        $instance = new static();
        add_action('add_meta_boxes', array($instance, 'add_meta_box_details_payment_xpay'));

        wp_enqueue_script('xpay-admin-checkout', plugins_url('assets/js/xpay-admin.js', WC_ECOMMERCE_GATEWAY_NEXI_MAIN_FILE), array('jquery'));

        wp_localize_script('xpay-admin-checkout', 'wpApiSettings', [
            'nonce' => wp_create_nonce('wp_rest'),
        ]);

        wp_enqueue_style('xpay-admin-checkout', plugins_url('assets/css/xpay-admin.css', WC_ECOMMERCE_GATEWAY_NEXI_MAIN_FILE));

        self::migrate_data();
    }

    public static function migrate_data()
    {
        $nexi_xpay_data_version = get_option("nexi_xpay_data_version");

        switch ($nexi_xpay_data_version) {
            case "":
                self::migrate_to_v1();
                return true;
                break;
            case "1":
                // Current version
                return true;
                break;

            default:
                return false;
                break;
        }
    }

    private static function migrate_to_v1()
    {
        $settings_key = "woocommerce_" . WC_GATEWAY_XPAY_PLUGIN_VARIANT . "_settings";

        Log::actionInfo("updating settings to v1");

        $currentConfig = get_option($settings_key);

        $currentConfig["nexi_xpay_alias"] = $currentConfig["cartasi_alias"];
        $currentConfig["nexi_xpay_mac"] = $currentConfig["cartasi_mac"];

        $currentConfig["nexi_xpay_test_mode"] = $currentConfig["cartasi_modalita_test"];

        $currentConfig["nexi_xpay_language"] = $currentConfig["cartasi_form_language"];
        $currentConfig["nexi_xpay_accounting"] = $currentConfig["contabilizzazione"];

        $currentConfig["nexi_xpay_oneclick_enabled"] = $currentConfig["abilita_modulo_oneclick"];

        $currentConfig["nexi_xpay_recurring_enabled"] = $currentConfig["abilita_modulo_ricorrenze"];
        $currentConfig["nexi_xpay_recurring_alias"] = $currentConfig["cartasi_alias_rico"];
        $currentConfig["nexi_xpay_recurring_mac"] = $currentConfig["cartasi_mac_rico"];
        $currentConfig["nexi_xpay_group"] = $currentConfig["gruppo_rico"];

        $currentConfig["nexi_xpay_3ds20_enabled"] = $currentConfig['enabled3ds'];

        update_option($settings_key, $currentConfig);
        Log::actionInfo("updated settings to v1");

        update_option('nexi_xpay_data_version', "1");
    }

    public function add_meta_box_details_payment_xpay()
    {
        $order_id = get_post_field("ID");
        $order = wc_get_order($order_id);
        if (!$order) {
            return;
        }

        $transactionCodTrans = get_xpay_post_meta($order_id, 'codTrans');
        if ($transactionCodTrans == "") {
            return;
        }

        if ($order->get_payment_method() === 'xpay' || substr($order->get_payment_method(), 0, 5) == 'xpay_') {
            add_meta_box('xpay-subscription-box', __('XPay payment details', 'woocommerce-gateway-nexi-xpay'), array($this, 'details_payment_xpay'), 'shop_order', 'normal', 'high');
        }
    }

    private function getCurrencySign($currency)
    {
        $currencySign = array(
            'EUR' => '&euro;',
            'CZK' => '&#75;&#269;',
            'PLN' => '&#122;&#322;',
            'NZD' => '&dollar;',
            'AUD' => '&dollar;'
        );

        return $currencySign[$currency];
    }

    private function getCurrencyLabel($currency)
    {
        $currencySign = array(
            'EUR' => __('Euros', 'woocommerce-gateway-nexi-xpay'),
            'CZK' => __('Czech Kurun', 'woocommerce-gateway-nexi-xpay'),
            'PLN' => __('Zloty', 'woocommerce-gateway-nexi-xpay'),
            'NZD' => __('Dollars', 'woocommerce-gateway-nexi-xpay'),
            'AUD' => __('Dollars', 'woocommerce-gateway-nexi-xpay')
        );

        return $currencySign[$currency];
    }

    public function details_payment_xpay()
    {
        if (!extension_loaded('bcmath')) {
?>
            <div id="message" class="error inline">
                <p><strong><?php echo __('Attention, bcmath PHP extension is not loaded, it is needed to list correctly order information and perform other operations.', 'woocommerce-gateway-nexi-xpay') ?></strong></p>
            </div>
        <?php
        }

        $order_id = get_post_field("ID");
        $transactionCodTrans = get_xpay_post_meta($order_id, 'codTrans');

        if ($transactionCodTrans == "") {
            echo __("Missing codTrans", 'woocommerce-gateway-nexi-xpay');
            return;
        }
        $transactionNumContratto = get_xpay_post_meta($order_id, 'num_contratto');
        $paymentCardBrandExpirationRaw = get_xpay_post_meta($order_id, 'scadenza_pan');

        $order = wc_get_order($order_id);

        $paymentCardBrandExpiration = "";
        if ($paymentCardBrandExpirationRaw != "") {
            $parsed = \DateTime::createFromFormat("Ym", $paymentCardBrandExpirationRaw);
            if ($parsed != null) {
                $paymentCardBrandExpiration = $parsed->format("m/Y");
            }
        }

        $canAccount = false;
        $currencySign = "";
        $currencyLabel = "";
        $operazioni = array();
        $transactionValue = "";
        $paymentCardBrandPan = "";
        $paymentCardBrandNazionalita = "";
        $custumerEmail = "";
        $transactionStatus = "";
        $transactionDate = "";
        $name = "";
        $cognome = "";

        try {
            $xpayOrderStatus = WC_Gateway_XPay_API::getInstance()->order_detail($transactionCodTrans);

            if (isset($xpayOrderStatus["stato"])) {
                $transactionStatus = $xpayOrderStatus["stato"];

                if ($xpayOrderStatus['stato'] == "Contabilizzato Parz.") {
                    $canAccount = true;
                } else if ($xpayOrderStatus['stato'] == "Autorizzato") {
                    $canAccount = true;
                }
            }

            if (isset($xpayOrderStatus["divisa"])) {
                $currencySign = self::getCurrencySign($xpayOrderStatus['divisa']);
                $currencyLabel = self::getCurrencyLabel($xpayOrderStatus['divisa']);
            }

            $transactionValue = div_bcdiv($xpayOrderStatus['importo'], 100);
            $paymentCardBrandPan = $xpayOrderStatus['pan'];
            $paymentCardBrandNazionalita = $xpayOrderStatus["nazione"];
            $paymentCardBrand = $xpayOrderStatus["brand"];
            $custumerEmail = $xpayOrderStatus["dettaglio"][0]["mail"];

            $name = $xpayOrderStatus['dettaglio'][0]["nome"];
            $cognome = $xpayOrderStatus['dettaglio'][0]["cognome"];

            $parsed = \DateTime::createFromFormat("Y-m-d H:i:s", $xpayOrderStatus["dataTransazione"]);
            if ($parsed != null) {
                $transactionDate = $parsed->format("d/m/Y H:i");
            }

            $operazioni = $xpayOrderStatus['dettaglio'][0]['operazioni'];
        } catch (\Exception $exc) {
            Log::actionWarning($exc->getMessage());
        }

        $custumerDisplayName = trim($name . " " . $cognome);

        $canAccount = current_user_can('manage_woocommerce') && $canAccount;
        $accountUrl = get_rest_url(null, "woocommerce-gateway-nexi-xpay/process_account/" . $order->get_id());
        ?>

        <div id="order_xpay_details" class="panel">

            <div class="order_data_column_container">

                <div class="order_data_column">
                    <h3><?php echo __("Cardholder", 'woocommerce-gateway-nexi-xpay') ?></h3>
                    <p>
                        <?php if ($custumerDisplayName != "") { ?>
                            <strong><?php echo __("Name: ", 'woocommerce-gateway-nexi-xpay') ?></strong> <?php echo htmlentities($custumerDisplayName) ?> <br>
                        <?php } ?>

                        <?php if ($custumerEmail != '') { ?>
                            <strong>Mail:</strong> <?php echo htmlentities($custumerEmail) ?> <br>
                        <?php } ?>
                    </p>
                </div>

                <div class="order_data_column">
                    <h3><?php echo __("Card detail", 'woocommerce-gateway-nexi-xpay') ?></h3>
                    <p>
                        <?php if ($paymentCardBrand != '') { ?>
                            <strong><?php echo __("Card: ", 'woocommerce-gateway-nexi-xpay') ?></strong> <?php echo htmlentities($paymentCardBrand) ?> <br>
                        <?php } ?>
                        <?php if ($paymentCardBrandNazionalita != '') { ?>
                            <strong><?php echo __("Nationality: ", 'woocommerce-gateway-nexi-xpay') ?></strong> <?php echo htmlentities($paymentCardBrandNazionalita) ?> <br>
                        <?php } ?>
                        <?php if ($paymentCardBrandPan != '') { ?>
                            <strong><?php echo __("Card pan: ", 'woocommerce-gateway-nexi-xpay') ?></strong> <?php echo htmlentities($paymentCardBrandPan) ?> <br>
                        <?php } ?>
                        <?php if ($paymentCardBrandExpiration != '') { ?>
                            <strong><?php echo __("Expiry date: ", 'woocommerce-gateway-nexi-xpay') ?></strong> <?php echo htmlentities($paymentCardBrandExpiration) ?> <br>
                        <?php } ?>
                    </p>
                </div>

                <div class="order_data_column">
                    <h3><?php echo __("Transaction detail", 'woocommerce-gateway-nexi-xpay') ?></h3>
                    <p>
                        <?php if ($transactionDate != '') { ?>
                            <strong><?php echo __("Date: ", 'woocommerce-gateway-nexi-xpay') ?></strong> <?php echo htmlentities($transactionDate) ?><br>
                        <?php } ?>
                        <?php if ($transactionValue) { ?>
                            <strong><?php echo __("Amount: ", 'woocommerce-gateway-nexi-xpay') ?></strong> <?php echo number_format($transactionValue, 2, ',', '.') . " " . $currencySign ?><br>
                        <?php } ?>
                        <?php if ($transactionCodTrans != '') { ?>
                            <strong> <?php echo __("Transaction code: ", 'woocommerce-gateway-nexi-xpay') ?></strong> <?php echo htmlentities($transactionCodTrans) ?><br>
                        <?php } ?>
                        <?php if ($transactionNumContratto != '') {
                        ?>
                            <strong> <?php echo __("Contract number: ", 'woocommerce-gateway-nexi-xpay') ?></strong> <?php echo htmlentities($transactionNumContratto) ?><br>
                        <?php } ?>
                        <?php if ($transactionStatus != '') { ?>
                            <strong><?php echo __("Status: ", 'woocommerce-gateway-nexi-xpay') ?> </strong><?php echo htmlentities($transactionStatus) ?><br>
                        <?php } ?>
                    </p>
                </div>
            </div>
            <?php
            $showOperations = count($operazioni) > 0;
            if ($showOperations || $canAccount) {
            ?>
                <?php if ($showOperations) { ?>
                    <h3><?php echo __("Accounting operations", 'woocommerce-gateway-nexi-xpay') ?></h3>
                <?php } else if ($canAccount) { ?>
                    <h3><?php echo __("New accounting operation", 'woocommerce-gateway-nexi-xpay') ?></h3>
                <?php } ?>
                <div class="woocommerce_subscriptions_related_orders operation-detail-container">
                    <?php if ($showOperations) { ?>
                        <div class="operation-detail-table">
                            <table class="wp-list-table widefat fixed striped table-view-list posts">
                                <thead>
                                    <tr>
                                        <th><?php echo __("Date", 'woocommerce-gateway-nexi-xpay') ?></th>
                                        <th><?php echo __("Type of operation", 'woocommerce-gateway-nexi-xpay') ?></th>
                                        <th><?php echo __("Amount", 'woocommerce-gateway-nexi-xpay') ?></th>
                                    </tr>
                                </thead>
                                <tbody>
                                    <?php foreach ($operazioni as $operazione) {
                                    ?>
                                        <tr>
                                            <td>
                                                <?php
                                                $oData = new \DateTime($operazione['dataOperazione']);
                                                echo $oData->format("d/m/Y H:i")
                                                ?>
                                            </td>
                                            <td><?php echo $operazione['tipoOperazione'] ?></td>
                                            <td><?php echo number_format(div_bcdiv($operazione['importo'], 100), 2, ",", ".") . ' ' . $this->getCurrencySign($operazione['divisa']) ?></td>
                                        </tr>

                                    <?php } ?>
                                </tbody>
                            </table>
                        </div>
                    <?php } ?>
                    <?php if ($canAccount && extension_loaded('bcmath')) { ?>
                        <div class="accounting-container">
                            <div class="accounting-action-container">
                                <div class="input-group">
                                    <input class="wc_input_price" id="xpay_account_account_input" value="" type="number" min="0.00" max="<?php echo number_format($transactionValue, 2, ".", ""); ?>" step="0.01">
                                    <div class="input-currency">
                                        <?php
                                        echo $currencySign;
                                        ?>
                                    </div>
                                </div>

                                <input type="hidden" id="xpay_account_form_api_url" value=" <?php echo $accountUrl . ''; ?>">
                                <input type="hidden" id="xpay_account_form_currency_label" value="<?php echo $currencyLabel . ''; ?>">
                                <input type="hidden" id="xpay_account_form_question" value=" <?php echo htmlentities(__('Do you confirm to account', 'woocommerce-gateway-nexi-xpay')); ?>">
                                <input type="hidden" id="xpay_account_form_success_message" value="<?php echo htmlentities(sprintf(__("Accounting of transaction %s successful", 'woocommerce-gateway-nexi-xpay'), $transactionCodTrans)) . ''; ?>">

                                <button type="button" id="xpay_account_form_btn" class="button button-primary accounting-btn">
                                    <?php echo __("Account", 'woocommerce-gateway-nexi-xpay') ?>
                                </button>
                            </div>
                        </div>
                    <?php } ?>
                </div>
            <?php } ?>
        </div>
<?php
    }
}
