<?php

/**
 * Copyright (c) 2019 Nexi Payments S.p.A.
 *
 * @author      iPlusService S.r.l.
 * @category    Payment Module
 * @package     Nexi XPay
 * @version     6.0.0
 * @copyright   Copyright (c) 2019 Nexi Payments S.p.A. (https://ecommerce.nexi.it)
 * @license     GNU General Public License v3.0
 */

namespace Nexi;

class WC_Gateway_XPay_API extends \WC_Settings_API
{

    private static $instance = null;

    /**
     * 
     * @return \Nexi\WC_Gateway_XPay_API
     */
    public static function getInstance()
    {
        if (self::$instance == null) {
            self::$instance = new static();
        }

        return self::$instance;
    }

    public $id = WC_GATEWAY_XPAY_PLUGIN_VARIANT;
    private $nexi_xpay_alias;
    private $nexi_xpay_mac;
    private $nexi_xpay_group = "";
    private $nexi_xpay_recurring_alias = "";
    private $nexi_xpay_recurring_mac = "";
    private $base_url = null;
    private $nexi_xpay_language;
    private $nexi_xpay_accounting;
    private $nexi_xpay_oneclick_enabled;
    private $nexi_xpay_recurring_enabled;
    private $nexi_xpay_3ds20_enabled;
    private $xpay_build_enviroment;

    private function __construct()
    {
        $this->init_settings();

        if ($this->settings["nexi_xpay_test_mode"] == "no") {
            $this->base_url = 'https://ecommerce.nexi.it/';
            $this->xpay_build_enviroment = "PROD";
        } else if (WC_GATEWAY_XPAY_PLUGIN_COLL) {
            $this->base_url = 'https://coll-ecommerce.nexi.it/';
            $this->xpay_build_enviroment = "INTEG";
        } else {
            $this->base_url = 'https://int-ecommerce.nexi.it/';
            $this->xpay_build_enviroment = "INTEG";
        }

        $this->nexi_xpay_alias = $this->settings["nexi_xpay_alias"];
        $this->nexi_xpay_mac = $this->settings["nexi_xpay_mac"];

        $this->nexi_xpay_language = $this->settings["nexi_xpay_language"];
        $this->nexi_xpay_accounting = $this->settings["nexi_xpay_accounting"];

        $this->nexi_xpay_oneclick_enabled = ($this->settings["nexi_xpay_oneclick_enabled"] == "yes");

        $this->nexi_xpay_recurring_enabled = ($this->settings["nexi_xpay_recurring_enabled"] == "yes");
        $this->nexi_xpay_recurring_alias = $this->settings["nexi_xpay_recurring_alias"];
        $this->nexi_xpay_recurring_mac = $this->settings["nexi_xpay_recurring_mac"];
        $this->nexi_xpay_group = $this->settings["nexi_xpay_group"] ?? '';

        $this->nexi_xpay_3ds20_enabled = ($this->settings['nexi_xpay_3ds20_enabled'] == 'yes');
    }

    public function get_profile_info()
    {
        Log::actionInfo(__FUNCTION__);
        $timeStamp = (time()) * 1000;

        // MAC calculation
        $macStr = 'apiKey=' . $this->nexi_xpay_alias;
        $macStr .= 'timeStamp=' . $timeStamp;
        $macStr .= $this->nexi_xpay_mac;
        $mac = sha1($macStr);

        // Params
        $payload = array(
            'apiKey' => $this->nexi_xpay_alias,
            'timeStamp' => $timeStamp,
            'mac' => $mac,
            'platform' => 'woocommerce',
            'platformVers' => WC_WOOCOMMERCE_GATEWAY_NEXI_XPAY_WOOCOMMERCE_VERSION,
            'pluginVers' => WC_GATEWAY_XPAY_VERSION
        );

        $profile_info = $this->exec_curl_post_json("ecomm/api/profileInfo", $payload);

        // Check on the outcome
        if ($profile_info['esito'] != 'OK') {
            Log::actionWarning(__FUNCTION__ . ": remote error: " . $profile_info['errore']['messaggio']);
            throw new \Exception(__('Response KO', 'woocommerce-gateway-nexi-xpay'));
        }

        $macResponseStr = 'esito=' . $profile_info['esito'];
        $macResponseStr .= 'idOperazione=' . $profile_info['idOperazione'];
        $macResponseStr .= 'timeStamp=' . $profile_info['timeStamp'];
        $macResponseStr .= $this->nexi_xpay_mac;

        $MACrisposta = sha1($macResponseStr);

        // Check on repsonse MAC
        if ($profile_info['mac'] != $MACrisposta) {
            Log::actionWarning(__FUNCTION__ . ": error: " . $profile_info['mac'] . " != " . $MACrisposta);
            throw new \Exception(__('Mac verification failed', 'woocommerce-gateway-nexi-xpay'));
        }

        update_option('xpay_available_methods', json_encode($profile_info['availableMethods']));
        update_option('xpay_logo_small', $profile_info['urlLogoNexiSmall']);
        update_option('xpay_logo_large', $profile_info['urlLogoNexiLarge']);


        if (is_array($profile_info['availableMethods'])) {
            $time = time();
            foreach ($profile_info['availableMethods'] as $method) {
                $option_name = 'xpay_apm_seen_' . $method['code'];
                if (!is_numeric(get_option($option_name))) {
                    $config_name = "woocommerce_xpay_" . $method['code'] . "_settings";
                    $config = get_option($config_name, array());
                    if ('yes' != $config['enabled']) {
                        $config['enabled'] = 'yes';
                        update_option($config_name, $config);
                    }
                }
                update_option($option_name, $time);
            }
        }

        return $profile_info;
    }

    public function get_payment_form($order, $selectedcard, $recurringPaymentRequired)
    {
        Log::actionInfo(__FUNCTION__ . ": for order " . $order->get_id());
        $importo = bcmul($order->get_total(), 100);

        $params = array(
            'alias' => $this->nexi_xpay_alias,
            'importo' => $importo,
            'divisa' => $order->get_currency(),
            'mail' => $order->get_billing_email(),
            'url' => get_rest_url(null, "woocommerce-gateway-nexi-xpay/redirect/" . $order->get_id()), //returning URL
            'url_back' => get_rest_url(null, "woocommerce-gateway-nexi-xpay/cancel/" . $order->get_id()), //cancel URL
            'languageId' => $this->get_language_id(), //checkout page lang 
            'descrizione' => "WC Order: " . $order->get_order_number(),
            'urlpost' => get_rest_url(null, "woocommerce-gateway-nexi-xpay/s2s/" . $order->get_id()), //S2S notification URL
            'selectedcard' => $selectedcard,
            'TCONTAB' => $this->nexi_xpay_accounting,
            'Note1' => 'woocommerce',
            'Note2' => WC_WOOCOMMERCE_GATEWAY_NEXI_XPAY_WOOCOMMERCE_VERSION,
            'Note3' => WC_GATEWAY_XPAY_VERSION
        );

        $costumer_id = get_post_meta($order->get_id(), '_customer_user', true);

        if ($recurringPaymentRequired) {
            if (!$this->nexi_xpay_recurring_enabled) {
                Log::actionWarning(__FUNCTION__ . ": recurring payment for non recurring payment method");
                throw new \Exception("Recurring not enabled");
            }

            // Contract number
            $md5_hash = md5($costumer_id . "@" . $order->get_order_number() . "@" . get_option('nexi_unique'));
            $params['num_contratto'] = "RP" . base_convert($md5_hash, 16, 36);

            $params['tipo_servizio'] = "paga_multi"; // static param for recurring payments
            $params['tipo_richiesta'] = "PP"; //PP = First Payment
            $params['gruppo'] = $this->nexi_xpay_group;

            $params['codTrans'] = $this->get_cod_trans($order->get_order_number(), "");

            $macString = 'codTrans=' . $params['codTrans'];
            $macString .= 'divisa=' . $params['divisa'];
            $macString .= 'importo=' . $params['importo'];
        } else {
            // Is using CC and is logged in enable the "one click" payment
            if ($selectedcard == "CC" && is_user_logged_in() && $this->nexi_xpay_oneclick_enabled) {
                // This will store, on nexi sistems, the card data to speed up the payment process
                $params['codTrans'] = $this->get_cod_trans($order->get_id(), "");

                $md5_hash = md5($costumer_id . "@" . get_option('nexi_unique'));
                $params['num_contratto'] = "OC" . base_convert($md5_hash, 16, 36);
                $params['tipo_servizio'] = "paga_1click";

                // Oneclick
                $macString = 'codTrans=' . $params['codTrans'];
                $macString .= 'divisa=' . $params['divisa'];
                $macString .= 'importo=' . $params['importo'];
                $macString .= 'gruppo=' . $this->nexi_xpay_group;
                $macString .= 'num_contratto=' . $params['num_contratto'];
            } else {
                $params['codTrans'] = $this->get_cod_trans($order->get_id(), "");
                $macString = 'codTrans=' . $params['codTrans'];
                $macString .= 'divisa=' . $params['divisa'];
                $macString .= 'importo=' . $params['importo'];
            }
        }
        $params['mac'] = sha1($macString . $this->nexi_xpay_mac);

        if ($this->nexi_xpay_3ds20_enabled && $selectedcard == "CC") {
            $params = array_merge($params, WC_3DS20_Data_Provider::calculate_params($order));
        }

        if ($selectedcard == "PAGODIL") {
            $params = array_merge($params, WC_Pagodil_Data_Provider::calculate_params($order));
        }

        update_post_meta($order->get_id(), "_xpay_" . "codTrans", $params['codTrans']);

        return array(
            "target_url" => $this->base_url . "ecomm/ecomm/DispatcherServlet",
            "fields" => $params,
        );
    }

    public function get_payment_build_payload($total)
    {
        $importo = bcmul($total, 100);

        $transactionId = substr("BP-" . date('ysdim') . "-" . time(), 0, 30);

        $divisa = array(
            'EUR' => '978',
            'CZK' => '203',
            'PLN' => '985',
            'AUD' => '036',
            'NZD' => '554'
        )[get_woocommerce_currency()];

        $buildData = array(
            "amount" => $importo,
            "enviroment" => $this->xpay_build_enviroment,
            "apiKey" => $this->nexi_xpay_alias,
            "transactionId" => $transactionId,
            "divisa" => $divisa,
            "timestamp" => time() * 1000,
            "mac" => sha1("codTrans=" . $transactionId . "divisa=" . $divisa . "importo=" . $importo . $this->nexi_xpay_mac),
            "language" => $this->get_language_id(),
            "buildStyle" => "OK",
            "build_border_color_default" => "white",
            "build_border_color_error" => "OK",
        );

        return $buildData;
    }

    /**
     * Function to retrieve the language of checkout page, based off the module settings and user navigation language
     * 
     */
    private function get_language_id()
    {
        // Default language, it will be overridden by user one 
        $language_id = 'ENG';

        // If the choice is automatic, it takes the default value
        if ($this->nexi_xpay_language == "AUTO") {
            $locale = get_locale();
            switch ($locale) {

                case 'it_IT':
                    $language_id = 'ITA';
                    break;

                case 'ar':
                    $language_id = 'ARA';
                    break;

                case 'zh_CN':
                    $language_id = 'CHI';
                    break;

                case 'ru_RU':
                    $language_id = 'RUS';
                    break;

                case 'es_ES':
                    $language_id = 'SPA';
                    break;

                case 'fr_FR':
                    $language_id = 'FRA';
                    break;

                case 'de_DE':
                    $language_id = 'GER';
                    break;

                case 'ja':
                    $language_id = 'GER';
                    break;

                case 'pt_PT':
                    $language_id = 'POR';
                    break;

                case 'en_GB':
                case 'en_US':
                default:
                    $language_id = 'ENG';
                    break;
            }
        } else {
            $language_id = $this->nexi_xpay_language;
        }

        return $language_id;
    }

    /**
     * Return codTrans param for XPay gateway
     *
     * @param string $payment_type
     * @return string
     */
    protected function get_cod_trans($order_id, $payment_type)
    {
        $cod_trans = $order_id;

        switch ($payment_type) {
            case "PR":
                $cod_trans .= "-PR";
                break;
            default:
        }

        return substr($cod_trans . "-" . time(), 0, 30);
    }

    public function validate_return_mac($request_parameters)
    {
        $mac = sha1('codTrans=' . $request_parameters['codTrans'] .
            'esito=' . $request_parameters['esito'] .
            'importo=' . $request_parameters['importo'] .
            'divisa=' . $request_parameters['divisa'] .
            'data=' . $request_parameters['data'] .
            'orario=' . $request_parameters['orario'] .
            'codAut=' . $request_parameters['codAut'] .
            $this->nexi_xpay_mac);
        if ($mac == $request_parameters["mac"]) {
            return true;
        }

        Log::actionWarning(__FUNCTION__ . ": error: " . $request_parameters["mac"] . " != " . $mac);

        return false;
    }

    public function recurring_payment($num_contratto, $scadenza_pan, $amount_to_charge, $currency, $order)
    {
        Log::actionInfo(__FUNCTION__ . ": begin num_contratto = " . $num_contratto . " for " . $amount_to_charge . " " . $currency . " order " . $order->get_id());
        $newCodTrans = $this->get_cod_trans($order->get_id(), "PR");

        $importo = bcmul($amount_to_charge, 100); // 5000 = 50,00 EURO (NB: the amount HAVE to be in cents format, so 50,00 euro becomes 5000  )
        $divisa = array(
            'EUR' => '978',
            'CZK' => '203',
            'PLN' => '985',
            'AUD' => '036',
            'NZD' => '554'
        )[$currency];
        $timeStamp = (time()) * 1000;

        // MAC calculation
        $mac = sha1('apiKey=' . $this->nexi_xpay_alias . 'numeroContratto=' . $num_contratto . 'codiceTransazione=' . $newCodTrans . 'importo=' . $importo . "divisa=" . $divisa . "scadenza=" . $scadenza_pan . "timeStamp=" . $timeStamp . $this->nexi_xpay_mac);

        $payload = array(
            'apiKey' => $this->nexi_xpay_alias,
            'numeroContratto' => $num_contratto,
            'codiceTransazione' => $newCodTrans,
            'importo' => $importo,
            'divisa' => $divisa,
            'scadenza' => $scadenza_pan,
            'codiceGruppo' => $this->nexi_xpay_group,
            'timeStamp' => $timeStamp,
            'mac' => $mac,
            'parametriAggiuntivi' => array(
                'Note1' => 'woocommerce',
                'Note2' => WC_WOOCOMMERCE_GATEWAY_NEXI_XPAY_WOOCOMMERCE_VERSION,
                'Note3' => WC_GATEWAY_XPAY_VERSION,
                'TCONTAB' => $this->nexi_xpay_accounting
            )
        );

        $payment_data = $this->exec_curl_post_json("ecomm/api/recurring/pagamentoRicorrente", $payload);

        if ($payment_data['esito'] != 'OK') {
            Log::actionWarning(__FUNCTION__ . ": remote error: " . $payment_data['errore']['messaggio']);
            throw new \Exception(__($payment_data['errore']['messaggio'], 'woocommerce-gateway-nexi-xpay'));
        }

        // MAC calculation with the repsonse params
        $macCalculated = sha1('esito=' . $payment_data['esito'] . 'idOperazione=' .
            $payment_data['idOperazione'] . 'timeStamp=' .
            $payment_data['timeStamp'] . $this->nexi_xpay_mac);

        if ($macCalculated != $payment_data['mac']) {
            Log::actionWarning(__FUNCTION__ . ": mac error: " . $payment_data["mac"] . " != " . $mac);
            throw new \Exception(__('Error in the calculation of the return MAC parameter', 'woocommerce-gateway-nexi-xpay'));
        }

        return $newCodTrans;
    }

    public function refund($cod_trans, $amount, $currency)
    {
        $apiKey = $this->nexi_xpay_alias;
        $chiaveSegreta = $this->nexi_xpay_mac;
        $importo = bcmul($amount, 100);
        $divisa = array(
            'EUR' => '978',
            'CZK' => '203',
            'PLN' => '985',
            'AUD' => '036',
            'NZD' => '554'
        )[$currency];
        $timeStamp = (time()) * 1000;

        //  MAC calculation
        $mac = sha1('apiKey=' . $apiKey . 'codiceTransazione=' . $cod_trans . 'divisa=' . $divisa . 'importo=' . $importo . 'timeStamp=' . $timeStamp . $chiaveSegreta);

        // Params
        $payload = array(
            // Mandatory
            'apiKey' => $apiKey,
            'codiceTransazione' => $cod_trans,
            'importo' => $importo,
            'divisa' => $divisa,
            'timeStamp' => $timeStamp,
            'mac' => $mac,
        );

        $operation_info = $this->exec_curl_post_json("ecomm/api/bo/storna", $payload);

        $MACrisposta = sha1('esito=' . $operation_info['esito'] . 'idOperazione=' . $operation_info['idOperazione'] . 'timeStamp=' . $operation_info['timeStamp'] . $chiaveSegreta);

        // Check on the response MAC
        if ($operation_info['mac'] != $MACrisposta) {
            Log::actionWarning(__FUNCTION__ . ": mac error: " . $operation_info["mac"] . " != " . $MACrisposta);
            throw new \Exception(__('Error in the calculation of the return MAC parameter', 'woocommerce-gateway-nexi-xpay'));
        }

        // Check on the outcome of the operation
        if ($operation_info['esito'] != 'OK') {
            Log::actionWarning(__FUNCTION__ . ": remote error: " . $operation_info['errore']['messaggio']);
            throw new \Exception(__($operation_info['errore']['messaggio'], 'woocommerce-gateway-nexi-xpay'));
        }

        return true;
    }

    public function order_detail($cod_trans)
    {
        $apiKey = $this->nexi_xpay_alias;
        $chiaveSegreta = $this->nexi_xpay_mac;
        if (substr($cod_trans, strlen($cod_trans) - 3) == "-PR") {
            $apiKey = $this->nexi_xpay_recurring_alias;
            $chiaveSegreta = $this->nexi_xpay_recurring_mac;
        }

        $timeStamp = (time()) * 1000;

        //  MAC calculation
        $mac = sha1('apiKey=' . $apiKey . 'codiceTransazione=' . $cod_trans . 'timeStamp=' . $timeStamp . $chiaveSegreta);

        // Params
        $payload = array(
            // Mandatory
            'apiKey' => $apiKey,
            'codiceTransazione' => $cod_trans,
            'timeStamp' => $timeStamp,
            'mac' => $mac
        );

        $operation_info = $this->exec_curl_post_json("ecomm/api/bo/situazioneOrdine", $payload);

        $MACrisposta = sha1('esito=' . $operation_info['esito'] . 'idOperazione=' .
            $operation_info['idOperazione'] . 'timeStamp=' .
            $operation_info['timeStamp'] . $chiaveSegreta);

        // Check on the response MAC
        if ($operation_info['mac'] != $MACrisposta) {
            Log::actionWarning(__FUNCTION__ . ": mac error: " . $operation_info["mac"] . " != " . $MACrisposta);
            throw new \Exception(__('Error in the calculation of the return MAC parameter', 'woocommerce-gateway-nexi-xpay'));
        }

        // Check on the outcome of the operation
        if ($operation_info['esito'] != 'OK') {
            Log::actionWarning(__FUNCTION__ . ": remote error: " . $operation_info['errore']['messaggio']);
            throw new \Exception(__($operation_info['errore']['messaggio'], 'woocommerce-gateway-nexi-xpay'));
        }

        return $operation_info["report"][0];
    }

    public function account($cod_trans, $amount, $currency)
    {
        $apiKey = $this->nexi_xpay_alias;
        $chiaveSegreta = $this->nexi_xpay_mac;
        if (substr($cod_trans, strlen($cod_trans) - 3) == "-PR") {
            $apiKey = $this->nexi_xpay_recurring_alias;
            $chiaveSegreta = $this->nexi_xpay_recurring_mac;
        }

        Log::actionInfo(__FUNCTION__ . ": begin " . $cod_trans . " " . $amount . " " . $currency);
        $divisa = array(
            'EUR' => '978',
            'CZK' => '203',
            'PLN' => '985',
            'AUD' => '036',
            'NZD' => '554'
        )[$currency];

        $timeStamp = (time()) * 1000;

        //  MAC calculation
        $mac = sha1('apiKey=' . $apiKey . 'codiceTransazione=' . $cod_trans . 'divisa=' . $divisa . 'importo=' . $amount . 'timeStamp=' . $timeStamp . $chiaveSegreta);

        // Params
        $payload = array(
            // Mandatory
            'apiKey' => $apiKey,
            'codiceTransazione' => $cod_trans,
            'importo' => $amount,
            'divisa' => $divisa,
            'timeStamp' => $timeStamp,
            'mac' => $mac,
        );

        $operation_info = $this->exec_curl_post_json("ecomm/api/bo/contabilizza", $payload);

        $MACrisposta = sha1('esito=' . $operation_info['esito'] . 'idOperazione=' . $operation_info['idOperazione'] . 'timeStamp=' . $operation_info['timeStamp'] . $chiaveSegreta);

        // Check on the response MAC
        if ($operation_info['mac'] != $MACrisposta) {
            Log::actionWarning(__FUNCTION__ . ": mac error: " . $operation_info["mac"] . " != " . $MACrisposta);
            throw new \Exception(__('Error in the calculation of the return MAC parameter', 'woocommerce-gateway-nexi-xpay'));
        }

        // Check on the outcome of the operation
        if ($operation_info['esito'] != 'OK') {
            Log::actionWarning(__FUNCTION__ . ": remote error: " . $operation_info['errore']['messaggio']);
            throw new \Exception(__($operation_info['errore']['messaggio'], 'woocommerce-gateway-nexi-xpay'));
        }

        Log::actionInfo(__FUNCTION__ . ": success " . $cod_trans . " " . $amount . " " . $currency);
        return true;
    }

    private function exec_curl_post_json($path, $payload)
    {
        $connection = curl_init();

        if (!$connection) {
            throw new \Exception(__('Can\'t connect!', 'woocommerce-gateway-nexi-xpay'));
        }

        curl_setopt_array($connection, array(
            CURLOPT_URL => $this->base_url . $path,
            CURLOPT_HTTPHEADER => array('Content-Type: application/json'),
            CURLOPT_POST => 1,
            CURLOPT_POSTFIELDS => json_encode($payload),
            CURLOPT_RETURNTRANSFER => 1,
            CURLINFO_HEADER_OUT => true
        ));

        $response = curl_exec($connection);
        if ($response == false) {
            throw new \Exception(sprintf(__('CURL exec error: %s', 'woocommerce-gateway-nexi-xpay'), curl_error($connection)));
        }
        curl_close($connection);

        $payment_data = json_decode($response, true);

        if (!(is_array($payment_data) && json_last_error() === JSON_ERROR_NONE)) {
            throw new \Exception(__('JSON error', 'woocommerce-gateway-nexi-xpay'));
        }

        return $payment_data;
    }

    public function getUrlXpayBuildJS()
    {
        if (strlen($this->nexi_xpay_alias) > 0 && strlen($this->base_url) > 0) {
            return $this->base_url . 'ecomm/XPayBuild/js?alias=' . $this->nexi_xpay_alias;
        }

        return "javascript:alert('missing config')";
    }

    public function pagaNonceCreazioneContratto($codiceTransazione, $importo, $nonce, $divisa, $numeroContratto)
    {
        $timeStamp = (time()) * 1000;

        // Calcolare il mac
        $macString = 'apiKey=' . $this->nexi_xpay_alias
            . 'codiceTransazione=' . $codiceTransazione
            . 'importo=' . $importo
            . 'divisa=' . $divisa
            . 'xpayNonce=' . $nonce
            . 'timeStamp=' . $timeStamp . $this->nexi_xpay_mac;
        $mac = sha1($macString);

        $pay_load = array(
            'apiKey' => $this->nexi_xpay_alias,
            'codiceTransazione' => $codiceTransazione,
            'importo' => $importo,
            'divisa' => $divisa,
            'xpayNonce' => $nonce,
            'timeStamp' => $timeStamp,
            'numeroContratto' => $numeroContratto,
            'codiceGruppo' => $this->nexi_xpay_group,
            'mac' => $mac,
            'parametriAggiuntivi' => array(
                'TCONTAB' => $this->nexi_xpay_accounting,
                'Note1' => 'woocommerce',
                'Note2' => WC_WOOCOMMERCE_GATEWAY_NEXI_XPAY_WOOCOMMERCE_VERSION,
                'Note3' => WC_GATEWAY_XPAY_VERSION
            )
        );

        $operation_info = $this->exec_curl_post_json("ecomm/api/hostedPayments/pagaNonceCreazioneContratto", $pay_load);

        // Calcolo MAC di risposta
        $MACrisposta = sha1('esito=' . $operation_info['esito']
            . 'idOperazione=' . $operation_info['idOperazione']
            . 'timeStamp=' . $operation_info['timeStamp'] . $this->nexi_xpay_mac);

        // Check on the response MAC
        if ($operation_info['mac'] != $MACrisposta) {
            Log::actionWarning(__FUNCTION__ . ": mac error: " . $operation_info["mac"] . " != " . $MACrisposta);
            throw new \Exception(__('Error in the calculation of the return MAC parameter', 'woocommerce-gateway-nexi-xpay'));
        }

        // Check on the outcome of the operation
        if ($operation_info['esito'] != 'OK') {
            Log::actionWarning(__FUNCTION__ . ": remote error: " . $operation_info['errore']['messaggio']);
            throw new \Exception(__($operation_info['errore']['messaggio'], 'woocommerce-gateway-nexi-xpay'));
        }

        Log::actionInfo(__FUNCTION__ . ": success");
        return true;
    }

    public function get_build_alias()
    {
        return $this->nexi_xpay_alias;
    }

    public function pagaNonce(bool $standalonePayment, $codiceTransazione, $importo, $nonce, $divisa)
    {
        $timeStamp = (time()) * 1000;

        // Calcolo MAC
        $macString = 'apiKey=' . $this->nexi_xpay_alias
            . 'codiceTransazione=' . $codiceTransazione
            . 'importo=' . $importo
            . 'divisa=' . $divisa
            . 'xpayNonce=' . $nonce
            . 'timeStamp=' . $timeStamp . $this->nexi_xpay_mac;
        $mac = sha1($macString);

        $pay_load = array(
            'apiKey' => $this->nexi_xpay_alias,
            'codiceTransazione' => $codiceTransazione,
            'importo' => $importo,
            'divisa' => $divisa,
            'xpayNonce' => $nonce,
            'timeStamp' => $timeStamp,
            'mac' => $mac,
            'parametriAggiuntivi' => array(
                'TCONTAB' => $this->nexi_xpay_accounting,
                'Note1' => 'woocommerce',
                'Note2' => WC_WOOCOMMERCE_GATEWAY_NEXI_XPAY_WOOCOMMERCE_VERSION,
                'Note3' => WC_GATEWAY_XPAY_VERSION
            )
        );

        $uri = null;
        if ($standalonePayment) {
            $uri = "ecomm/api/hostedPayments/pagaNonce";
        } else {
            $uri = "ecomm/api/recurring/pagamentoRicorrente3DS";
        }

        $operation_info = $this->exec_curl_post_json($uri, $pay_load);

        // Calcolo MAC di risposta
        $MACrisposta = sha1('esito=' . $operation_info['esito']
            . 'idOperazione=' . $operation_info['idOperazione']
            . 'timeStamp=' . $operation_info['timeStamp'] . $this->nexi_xpay_mac);

        // Check on the response MAC
        if ($operation_info['mac'] != $MACrisposta) {
            Log::actionWarning(__FUNCTION__ . ": mac error: " . $operation_info["mac"] . " != " . $MACrisposta);
            throw new \Exception(__('Error in the calculation of the return MAC parameter', 'woocommerce-gateway-nexi-xpay'));
        }

        // Check on the outcome of the operation
        if ($operation_info['esito'] != 'OK') {
            Log::actionWarning(__FUNCTION__ . ": remote error: " . $operation_info['errore']['messaggio']);
            throw new \Exception(__($operation_info['errore']['messaggio'], 'woocommerce-gateway-nexi-xpay'));
        }

        Log::actionInfo(__FUNCTION__ . ": success");
        return true;
    }

    public function calculate_mac_for_build_oneclick($codTransCvv, $divisa, $importoCvv)
    {
        $divisa = array(
            'EUR' => '978',
            'CZK' => '203',
            'PLN' => '985',
            'AUD' => '036',
            'NZD' => '554'
        )[$divisa];

        return sha1('codTrans=' . $codTransCvv
            . 'divisa=' . $divisa
            . 'importo=' . $importoCvv
            . $this->nexi_xpay_mac);
    }
}
