<?php
/**
 * Copyright (c) 2019 Nexi Payments S.p.A.
 *
 * @author      iPlusService S.r.l.
 * @category    Payment Module
 * @package     Nexi XPay
 * @version     6.0.0
 * @copyright   Copyright (c) 2019 Nexi Payments S.p.A. (https://ecommerce.nexi.it)
 * @license     GNU General Public License v3.0
 */

namespace Nexi;

class WC_Gateway_XPay_Cards extends WC_Gateway_XPay_Generic_Method
{

    public function __construct()
    {
        parent::__construct('xpay', true);

        $this->method_title = __('Payment cards', 'woocommerce-gateway-nexi-xpay');
        $this->method_description = __('Payment gateway.', 'woocommerce-gateway-nexi-xpay');
        $this->title = $this->method_title;

        $avaiable_methods = json_decode(\WC_Admin_Settings::get_option('xpay_available_methods'), true);
        $img_list = "";

        if (is_array($avaiable_methods)) {
            foreach ($avaiable_methods as $am) {
                if ($am['type'] == "CC") {
                    $configuration = array(
                        'maestro' => 'padding-right:10px; padding-top:6px; padding-bottom:6px;',
                        'mastercard' => 'padding-right:10px; padding-top:6px; padding-bottom:6px;',
                        'visa' => 'padding-right:10px; padding-top:9px; padding-bottom:9px;',
                    );
                    if (isset($configuration[strtolower($am['code'])])) {
                        $style = $configuration[strtolower($am['code'])];
                    } else {
                        $style = "";
                    }

                    $img_list .= "<span style='display:inline-block; height:40px; float:left;" . $style . "'>"
                            . "<img src='" . $am['image'] . "'  style='height:100%;float: none;position: unset;display: block;'>"
                            . "</span>";
                }
            }
        }

        if ($img_list != "") {
            $img_list = "<span id='xpay_list_icon' style='margin-bottom:10px;width:auto;display:inline-block;'>" . $img_list . "</span><br>";
        }
        $this->description = $img_list . __("Pay securely by credit, debit and prepaid card through Nexi.", 'woocommerce-gateway-nexi-xpay');

        if (\WC_Admin_Settings::get_option('xpay_logo_small') == "") {
            $this->icon = WC_WOOCOMMERCE_GATEWAY_NEXI_XPAY_DEFAULT_LOGO_URL;
        } else {
            $this->icon = \WC_Admin_Settings::get_option('xpay_logo_small');
        }

        add_action('woocommerce_receipt_' . $this->id, array($this, 'exec_payment'));

        // Admin page
        add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_save'));

        add_action('woocommerce_scheduled_subscription_payment_' . $this->id, array($this, 'scheduled_subscription_payment'), 10, 2);

        $this->selectedCard = "CC";
    }

    public static function addNexiPendingPaymentOrderStatus()
    {
        register_post_status('wc-pd-pending-status', array(
            'label' => __('To be authorized', 'woocommerce-gateway-nexi-xpay'),
            'public' => true,
            'exclude_from_search' => false,
            'show_in_admin_all_list' => true,
            'show_in_admin_status_list' => true,
            'label_count' => _n_noop('To be authorized (%s)', 'To be authorized (%s)', 'woocommerce-gateway-nexi-xpay')
        ));
    }

    public static function wcOrderStatusesFilter($orderStatuses)
    {
        $orderStatuses['wc-pd-pending-status'] = __('To be authorized', 'woocommerce-gateway-nexi-xpay');

        return $orderStatuses;
    }

    public static function validOrderStatusesForPaymentCompleteFilter($orderStatuses)
    {
        $orderStatuses[] = 'pd-pending-status';

        return $orderStatuses;
    }

    public static function my_error_notice()
    {
        ?>
        <div class="error notice">
            <p><?php echo __('Invalid credentials. Check and try again.', 'woocommerce-gateway-nexi-xpay'); ?></p>
        </div>
        <?php
    }

    function process_admin_save()
    {
        $this->process_admin_options();

        self::update_profile_info();
    }

    public static function update_profile_info()
    {
        try {
            WC_Gateway_XPay_API::getInstance()->get_profile_info();
        } catch (\Exception $exc) {
            delete_option('xpay_available_methods');
            delete_option('xpay_logo_small');
            delete_option('xpay_logo_large');

            add_action('admin_notices', WC_Gateway_XPay_Cards::class . "::my_error_notice");

            Log::actionWarning($exc->getMessage());
        }
    }

    function init_form_fields()
    {
        parent::init_form_fields();

        $descriptionEnable = "";
        $descriptionEnable .= __("Enable Nexi XPay payment plugin.", 'woocommerce-gateway-nexi-xpay') . '<br>';
        $descriptionEnable .= '<br><p>' . __('For a correct behavior of the module, check in the configuration section of the Nexi back-office that the transaction cancellation in the event of a failed notification is set.', 'woocommerce-gateway-nexi-xpay') . '</p>';
        $descriptionEnable .= '<br><p>' . __('A POST notification by the Nexi servers is sent to the following address, containing information on the outcome of the payment.', 'woocommerce-gateway-nexi-xpay') . '<br><b>' . get_rest_url(null, "woocommerce-gateway-nexi-xpay/s2s/") . "[order id]" . '</b><br>' . __('The notification is essential for the functioning of the plugin, it is therefore necessary that it is not blocked or filtered by the site infrastructure.', 'woocommerce-gateway-nexi-xpay') . '</p>';

        $description3ds20 = "";
        $description3ds20 .= __('The 3D Secure 2 protocol adopted by the main international circuits (Visa, MasterCard, American Express), introduces new authentication methods, able to improve and speed up the cardholder\'s purchase experience.', 'woocommerce-gateway-nexi-xpay');
        $description3ds20 .= '<br><br>';
        $description3ds20 .= __('By activating this option it is established that the terms and conditions that you submit to your customers, with particular reference to the privacy policy,', 'woocommerce-gateway-nexi-xpay');
        $description3ds20 .= __(' are foreseen to include the acquisition and processing of additional data provided by the ');
        $description3ds20 .= '<a href=\"https://ecommerce.nexi.it/specifiche-tecniche/3dsecure2/introduzione.html\" target="_blank">3D Secure 2 Service</a>';
        $description3ds20 .= __(' (for example, shipping and / or invoicing address, payment details). ');
        $description3ds20 .= __('Nexi and the International Circuits use the additional data collected separately for the purpose of fraud prevention.', 'woocommerce-gateway-nexi-xpay');
        $this->form_fields = array(
            'title_section_1' => array(
                'title' => __("Payment plugin configuration", 'woocommerce-gateway-nexi-xpay'),
                'type' => 'title',
            ),
            'enabled' => array(
                'title' => __('Enable/Disable', 'woocommerce-gateway-nexi-xpay'),
                'type' => 'checkbox',
                'label' => $descriptionEnable,
                'default' => 'no',
            ),
            'nexi_xpay_language' => array(
                'title' => __('Language form', 'woocommerce-gateway-nexi-xpay'),
                'type' => 'select',
                'options' => $this->getOptionsConfigLanguage(),
                'desc_tip' => __('Select the language of the payment form', 'woocommerce-gateway-nexi-xpay'),
            ),
            'nexi_xpay_accounting' => array(
                'title' => __('Accounting', 'woocommerce-gateway-nexi-xpay'),
                'type' => 'select',
                'options' => array(
                    "C" => __('Immediate', 'woocommerce-gateway-nexi-xpay'),
                    "D" => __('Deferred', 'woocommerce-gateway-nexi-xpay')
                ),
                'desc_tip' => __('The field identifies the collection method that the merchant wants to apply to the single transaction, if valued with:<br>- I (immediate) the transaction if authorized is also collected without further intervention by the operator and without considering the default profile set on the terminal.<br>- D (deferred) or the field is not inserted the transaction if authorized is managed according to what is defined by the terminal profile', 'woocommerce-gateway-nexi-xpay'),
            ),
            'nexi_xpay_test_mode' => array(
                'title' => __('Enable/Disable TEST Mode', 'woocommerce-gateway-nexi-xpay'),
                'type' => 'checkbox',
                'label' => __('Enable Nexi XPay plugin in testing mode.', 'woocommerce-gateway-nexi-xpay'),
                'default' => 'no',
                'description' => __('Register on', 'woocommerce-gateway-nexi-xpay') . ' <a href=\'https://ecommerce.nexi.it/area-test\'>' . __('ecommerce.nexi.it/area-test', 'woocommerce-gateway-nexi-xpay') . '</a> ' . __('to have TEST\'s credentials.', 'woocommerce-gateway-nexi-xpay'),
            ),
            'title_section_2' => array(
                'title' => __("Gateway XPay configuration", 'woocommerce-gateway-nexi-xpay'),
                'type' => 'title',
            ),
            'nexi_xpay_3ds20_enabled' => array(
                'title' => __('Enable 3D Secure 2 service', 'woocommerce-gateway-nexi-xpay'),
                'type' => 'checkbox',
                'label' => __('Enable the sending of the fields for the 3D Secure 2 service', 'woocommerce-gateway-nexi-xpay'),
                'default' => 'no',
                'description' => $description3ds20,
            ),
            'nexi_xpay_alias' => array(
                'title' => __('Alias', 'woocommerce-gateway-nexi-xpay'),
                'type' => 'text',
                'desc_tip' => __('Given to Merchant by Nexi.', 'woocommerce-gateway-nexi-xpay'),
            ),
            'nexi_xpay_mac' => array(
                'title' => __('Key MAC', 'woocommerce-gateway-nexi-xpay'),
                'type' => 'text',
                'desc_tip' => __('Given to Merchant by Nexi.', 'woocommerce-gateway-nexi-xpay'),
            ),
            'title_section_3' => array(
                'title' => __("OneClick checkout configuration", 'woocommerce-gateway-nexi-xpay'),
                'type' => 'title',
            ),
            'nexi_xpay_oneclick_enabled' => array(
                'title' => __('Enable/Disable OneClick', 'woocommerce-gateway-nexi-xpay'),
                'type' => 'checkbox',
                'label' => __('Enable Nexi XPay for OneClick payment', 'woocommerce-gateway-nexi-xpay'),
                'default' => 'no',
                'description' => '',
            ),
        );

        $this->form_fields = array_merge($this->form_fields, \Nexi\WC_Pagodil_Configuration::getSettingsForm());

        if (function_exists("wcs_is_subscription")) {
            $this->form_fields = array_merge($this->form_fields, array(
                'title_section_6' => array(
                    'title' => __("Subscription configuration", 'woocommerce-gateway-nexi-xpay'),
                    'type' => 'title',
                ),
                'nexi_xpay_recurring_enabled' => array(
                    'title' => __('Enable/Disable Recurring', 'woocommerce-gateway-nexi-xpay'),
                    'type' => 'checkbox',
                    'label' => __("Enable Nexi XPay for subscription's payment", 'woocommerce-gateway-nexi-xpay'),
                    'default' => 'no',
                    'description' => '',
                ),
                'nexi_xpay_recurring_alias' => array(
                    'title' => __('Recurring Alias', 'woocommerce-gateway-nexi-xpay'),
                    'type' => 'text',
                    'desc_tip' => __('Given to Merchant by Nexi.', 'woocommerce-gateway-nexi-xpay'),
                ),
                'nexi_xpay_recurring_mac' => array(
                    'title' => __('Recurring key MAC', 'woocommerce-gateway-nexi-xpay'),
                    'type' => 'text',
                    'desc_tip' => __('Given to Merchant by Nexi.', 'woocommerce-gateway-nexi-xpay'),
                ),
                'nexi_xpay_group' => array(
                    'title' => __('Group', 'woocommerce-gateway-nexi-xpay'),
                    'type' => 'text',
                    'desc_tip' => __('Given to Merchant by Nexi.', 'woocommerce-gateway-nexi-xpay'),
                ),
            ));
        }
    }

    /**
     * List of languages, shown in configuration
     * 
     * @return array
     */
    private function getOptionsConfigLanguage()
    {
        return array(
            "AUTO" => __("Automatic"),
            "ITA" => __("Italian"),
            "ARA" => __("Arabic"),
            "CHI" => __("Chinese"),
            "ENG" => __("English"),
            "RUS" => __("Russian"),
            "SPA" => __("Spanish"),
            "POR" => __("Portuguese"),
            "FRA" => __("French"),
            "GER" => __("German"),
            "JPG" => __("Japanese"),
        );
    }

}
