<?php

/**
 * Copyright (c) 2019 Nexi Payments S.p.A.
 *
 * @author      iPlusService S.r.l.
 * @category    Payment Module
 * @package     Nexi XPay
 * @version     6.0.0
 * @copyright   Copyright (c) 2019 Nexi Payments S.p.A. (https://ecommerce.nexi.it)
 * @license     GNU General Public License v3.0
 */

namespace Nexi;

class WC_Gateway_XPay_Cards_Build extends WC_Gateway_XPay_Generic_Method
{

    public function __construct()
    {
        parent::__construct('xpay_build', true);

        $this->method_title = __('Payment cards', 'woocommerce-gateway-nexi-xpay');
        $this->method_description = __('Payment gateway.', 'woocommerce-gateway-nexi-xpay');
        $this->title = $this->method_title;

        $this->description = __("Pay securely by credit, debit and prepaid card through Nexi.", 'woocommerce-gateway-nexi-xpay');

        if (\WC_Admin_Settings::get_option('xpay_logo_small') == "") {
            $this->icon = WC_WOOCOMMERCE_GATEWAY_NEXI_XPAY_DEFAULT_LOGO_URL;
        } else {
            $this->icon = \WC_Admin_Settings::get_option('xpay_logo_small');
        }

        add_action('woocommerce_receipt_' . $this->id, array($this, 'exec_payment'));

        // Admin page
        add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_save'));

        add_action('woocommerce_scheduled_subscription_payment_' . $this->id, array($this, 'scheduled_subscription_payment'), 10, 2);

        $this->selectedCard = "CC";

        wp_enqueue_script('xpay_build_lib', \Nexi\WC_Gateway_XPay_API::getInstance()->getUrlXpayBuildJS());
    }

    public static function my_error_notice()
    {
?>
        <div class="error notice">
            <p><?php echo __('Invalid credentials. Check and try again.', 'woocommerce-gateway-nexi-xpay'); ?></p>
        </div>
    <?php
    }

    function process_admin_save()
    {
        $this->process_admin_options();

        self::update_profile_info();
    }

    public static function update_profile_info()
    {
        try {
            WC_Gateway_XPay_API::getInstance()->get_profile_info();
        } catch (\Exception $exc) {
            delete_option('xpay_available_methods');
            delete_option('xpay_logo_small');
            delete_option('xpay_logo_large');

            add_action('admin_notices', WC_Gateway_XPay_Cards_Build::class . "::my_error_notice");

            Log::actionWarning($exc->getMessage());
        }
    }

    function init_form_fields()
    {
        parent::init_form_fields();

        $descriptionEnable = "";
        $descriptionEnable .= __("Enable Nexi XPay payment plugin.", 'woocommerce-gateway-nexi-xpay') . '<br>';
        $descriptionEnable .= '<br><p>' . __('For a correct behavior of the module, check in the configuration section of the Nexi back-office that the transaction cancellation in the event of a failed notification is set.', 'woocommerce-gateway-nexi-xpay') . '</p>';
        $descriptionEnable .= '<br><p>' . __('A POST notification by the Nexi servers is sent to the following address, containing information on the outcome of the payment.', 'woocommerce-gateway-nexi-xpay') . '<br><b>' . get_rest_url(null, "woocommerce-gateway-nexi-xpay/s2s/") . "[order id]" . '</b><br>' . __('The notification is essential for the functioning of the plugin, it is therefore necessary that it is not blocked or filtered by the site infrastructure.', 'woocommerce-gateway-nexi-xpay') . '</p>';

        $description3ds20 = "";
        $description3ds20 .= __('The 3D Secure 2 protocol adopted by the main international circuits (Visa, MasterCard, American Express), introduces new authentication methods, able to improve and speed up the cardholder\'s purchase experience.', 'woocommerce-gateway-nexi-xpay');
        $description3ds20 .= '<br><br>';
        $description3ds20 .= __('By activating this option it is established that the terms and conditions that you submit to your customers, with particular reference to the privacy policy,', 'woocommerce-gateway-nexi-xpay');
        $description3ds20 .= __(' are foreseen to include the acquisition and processing of additional data provided by the ');
        $description3ds20 .= '<a href=\"https://ecommerce.nexi.it/specifiche-tecniche/3dsecure2/introduzione.html\" target="_blank">3D Secure 2 Service</a>';
        $description3ds20 .= __(' (for example, shipping and / or invoicing address, payment details). ');
        $description3ds20 .= __('Nexi and the International Circuits use the additional data collected separately for the purpose of fraud prevention.', 'woocommerce-gateway-nexi-xpay');
        $this->form_fields = array(
            'title_section_1' => array(
                'title' => __("Payment plugin configuration", 'woocommerce-gateway-nexi-xpay'),
                'type' => 'title',
            ),
            'enabled' => array(
                'title' => __('Enable/Disable', 'woocommerce-gateway-nexi-xpay'),
                'type' => 'checkbox',
                'label' => $descriptionEnable,
                'default' => 'no'
            ),
            'nexi_xpay_language' => array(
                'title' => __('Language form', 'woocommerce-gateway-nexi-xpay'),
                'type' => 'select',
                'options' => $this->getOptionsConfigLanguage(),
                'desc_tip' => __('Select the language of the payment form', 'woocommerce-gateway-nexi-xpay')
            ),
            'nexi_xpay_accounting' => array(
                'title' => __('Accounting', 'woocommerce-gateway-nexi-xpay'),
                'type' => 'select',
                'options' => array(
                    "C" => __('Immediate', 'woocommerce-gateway-nexi-xpay'),
                    "D" => __('Deferred', 'woocommerce-gateway-nexi-xpay'),
                ),
                'desc_tip' => __('The field identifies the collection method that the merchant wants to apply to the single transaction, if valued with:<br>- I (immediate) the transaction if authorized is also collected without further intervention by the operator and without considering the default profile set on the terminal.<br>- D (deferred) or the field is not inserted the transaction if authorized is managed according to what is defined by the terminal profile', 'woocommerce-gateway-nexi-xpay'),
            ),
            'nexi_xpay_test_mode' => array(
                'title' => __('Enable/Disable TEST Mode', 'woocommerce-gateway-nexi-xpay'),
                'type' => 'checkbox',
                'label' => __('Enable Nexi XPay plugin in testing mode.', 'woocommerce-gateway-nexi-xpay'),
                'default' => 'no',
                'description' => __('Register on', 'woocommerce-gateway-nexi-xpay') . ' <a href=\'https://ecommerce.nexi.it/area-test\'>' . __('ecommerce.nexi.it/area-test', 'woocommerce-gateway-nexi-xpay') . '</a> ' . __('to have TEST\'s credentials.', 'woocommerce-gateway-nexi-xpay'),
            ),
            'title_section_2' => array(
                'title' => __("Gateway XPay configuration", 'woocommerce-gateway-nexi-xpay'),
                'type' => 'title',
            ),
            'nexi_xpay_3ds20_enabled' => array(
                'title' => __('Enable 3D Secure 2 service', 'woocommerce-gateway-nexi-xpay'),
                'type' => 'checkbox',
                'label' => __('Enable the sending of the fields for the 3D Secure 2 service', 'woocommerce-gateway-nexi-xpay'),
                'default' => 'no',
                'description' => $description3ds20,
            ),
            'nexi_xpay_alias' => array(
                'title' => __('Alias', 'woocommerce-gateway-nexi-xpay'),
                'type' => 'text',
                'desc_tip' => __('Given to Merchant by Nexi.', 'woocommerce-gateway-nexi-xpay'),
            ),
            'nexi_xpay_mac' => array(
                'title' => __('Key MAC', 'woocommerce-gateway-nexi-xpay'),
                'type' => 'text',
                'desc_tip' => __('Given to Merchant by Nexi.', 'woocommerce-gateway-nexi-xpay'),
            ),
            'title_section_3' => array(
                'title' => __("OneClick checkout configuration", 'woocommerce-gateway-nexi-xpay'),
                'type' => 'title',
            ),
            'nexi_xpay_oneclick_enabled' => array(
                'title' => __('Enable/Disable OneClick', 'woocommerce-gateway-nexi-xpay'),
                'type' => 'checkbox',
                'label' => __('Enable Nexi XPay for OneClick payment', 'woocommerce-gateway-nexi-xpay'),
                'default' => 'no',
                'description' => '',
            ),
            'style_title' => array(
                'title' => __('Style configuration', 'woocommerce-gateway-nexi-xpay'),
                'type' => 'title',
                'description' => __('By using this configurator you can change the look and feel of your module', 'woocommerce-gateway-nexi-xpay'),
                'class' => 'style_title',
            ),
            'preview' => array(
                'title' => __('Preview', 'woocommerce-gateway-nexi-xpay'),
                'type' => 'label',
                'label' => '',
            ),
            'font_family' => array(
                'title' => __('Font family', 'woocommerce-gateway-nexi-xpay'),
                'type' => 'text',
                'description' => __('The font family in the CC Form', 'woocommerce-gateway-nexi-xpay'),
                'default' => 'Arial',
                'desc_tip' => true,
                'class' => 'build_style font-family',
            ),
            'font_size' => array(
                'title' => __('Font size', 'woocommerce-gateway-nexi-xpay'),
                'type' => 'text',
                'description' => __('The size of the font in the CC Form in pixel', 'woocommerce-gateway-nexi-xpay'),
                'default' => '15px',
                'desc_tip' => true,
                'class' => 'build_style font-size',
            ),
            'font_style' => array(
                'title' => __('Font style', 'woocommerce-gateway-nexi-xpay'),
                'type' => 'select',
                'description' => __('Font style in the CC Form', 'woocommerce-gateway-nexi-xpay'),
                'default' => 'Normal',
                'desc_tip' => true,
                'options' => $this->getOptionsConfigFontStyle(),
                'class' => 'build_style font-style',
            ),
            'font_variant' => array(
                'title' => __('Font variant', 'woocommerce-gateway-nexi-xpay'),
                'type' => 'select',
                'description' => __('Font variant in the CC Form', 'woocommerce-gateway-nexi-xpay'),
                'default' => 'Normal',
                'desc_tip' => true,
                'options' => $this->getOptionsConfigFontVariant(),
                'class' => 'build_style font-variant',
            ),
            'letter_spacing' => array(
                'title' => __('Letter spacing', 'woocommerce-gateway-nexi-xpay'),
                'type' => 'text',
                'description' => __('The space between letters in pixel', 'woocommerce-gateway-nexi-xpay'),
                'default' => '1px',
                'desc_tip' => true,
                'class' => 'build_style letter-spacing',
            ),
            'border_color_ok' => array(
                'title' => __('Border Color', 'woocommerce-gateway-nexi-xpay'),
                'type' => 'color',
                'description' => __('When form is empty or correct', 'woocommerce-gateway-nexi-xpay'),
                'default' => '#CDCDCD',
                'desc_tip' => true,
                'css' => 'width:370px;',
                'class' => 'build_style border-color',
            ),
            'border_color_ko' => array(
                'title' => __('Error Border Color', 'woocommerce-gateway-nexi-xpay'),
                'type' => 'color',
                'description' => __('When form has error', 'woocommerce-gateway-nexi-xpay'),
                'default' => '#C80000',
                'desc_tip' => true,
                'css' => 'width:370px;',
            ),
            'placeholder_color' => array(
                'title' => __('Placeholder Color', 'woocommerce-gateway-nexi-xpay'),
                'type' => 'color',
                'description' => __('Text color of placeholder', 'woocommerce-gateway-nexi-xpay'),
                'default' => '#CDCDCD',
                'desc_tip' => true,
                'css' => 'width:370px;',
                'class' => 'build_style placeholder-color',
            ),
            'text_color' => array(
                'title' => __('Text Color', 'woocommerce-gateway-nexi-xpay'),
                'type' => 'color',
                'description' => __('Text color in input field', 'woocommerce-gateway-nexi-xpay'),
                'default' => '#5C5C5C',
                'desc_tip' => true,
                'css' => 'width:370px;',
                'class' => 'build_style color',
            ),
        );

        $this->form_fields = array_merge($this->form_fields, \Nexi\WC_Pagodil_Configuration::getSettingsForm());

        if (function_exists("wcs_is_subscription")) {
            $this->form_fields = array_merge($this->form_fields, array(
                'title_section_4' => array(
                    'title' => __("Subscription configuration", 'woocommerce-gateway-nexi-xpay'),
                    'type' => 'title',
                ),
                'nexi_xpay_recurring_enabled' => array(
                    'title' => __('Enable/Disable Recurring', 'woocommerce-gateway-nexi-xpay'),
                    'type' => 'checkbox',
                    'label' => __("Enable Nexi XPay for subscription's payment", 'woocommerce-gateway-nexi-xpay'),
                    'default' => 'no',
                    'description' => '',
                ),
                'nexi_xpay_recurring_alias' => array(
                    'title' => __('Recurring Alias', 'woocommerce-gateway-nexi-xpay'),
                    'type' => 'text',
                    'desc_tip' => __('Given to Merchant by Nexi.', 'woocommerce-gateway-nexi-xpay'),
                ),
                'nexi_xpay_recurring_mac' => array(
                    'title' => __('Recurring key MAC', 'woocommerce-gateway-nexi-xpay'),
                    'type' => 'text',
                    'desc_tip' => __('Given to Merchant by Nexi.', 'woocommerce-gateway-nexi-xpay'),
                ),
                'nexi_xpay_group' => array(
                    'title' => __('Group', 'woocommerce-gateway-nexi-xpay'),
                    'type' => 'text',
                    'desc_tip' => __('Given to Merchant by Nexi.', 'woocommerce-gateway-nexi-xpay'),
                ),
            ));
        }
    }

    private function getOptionsConfigLanguage()
    {
        if (get_locale() == 'it_IT') {
            $LANG_IT = "Italiano";
            $LANG_EN = "Inglese";
            $LANG_SP = "Spagnolo";
            $LANG_PR = "Portoghese";
            $LANG_FR = "Francese";
            $LANG_DE = "Tedesco";
            $LANG_JP = "Giapponese";
            $LANG_AR = "Arabo";
            $LANG_CH = "Cinese";
            $LANG_RU = "Russo";
            $LANG_AUTO = "Automatico";
        } else {
            $LANG_IT = "Italian";
            $LANG_EN = "English";
            $LANG_SP = "Spanish";
            $LANG_PR = "Portuguese";
            $LANG_FR = "Franch";
            $LANG_DE = "German";
            $LANG_JP = "Japanese";
            $LANG_AR = "Arabic";
            $LANG_CH = "Chinese";
            $LANG_RU = "Russian";
            $LANG_AUTO = "Automatic";
        }

        $nexi_xpay_language_ids = array(
            "AUTO" => $LANG_AUTO,
            "ITA" => $LANG_IT,
            "ARA" => $LANG_AR,
            "CHI" => $LANG_CH,
            "ENG" => $LANG_EN,
            "RUS" => $LANG_RU,
            "SPA" => $LANG_SP,
            "POR" => $LANG_PR,
            "FRA" => $LANG_FR,
            "GER" => $LANG_DE,
            "JPG" => $LANG_JP,
        );

        return $nexi_xpay_language_ids;
    }

    /**
     * Generate build preview HTML.
     *
     * @param mixed $key
     * @param mixed $data
     * @return string
     */
    public function generate_label_html($key, $data)
    {
        $field = $this->plugin_id . $this->id . '_' . $key;

        ob_start();
    ?>
        <tr valign="top">
            <th scope="row" class="titledesc">
                <label for="<?php echo esc_attr($field); ?>"><?php echo wp_kses_post($data['title']); ?></label>
            </th>
            <td class="forminp">
                <fieldset>
                    <?php
                    $path = plugin_dir_path(WC_ECOMMERCE_GATEWAY_NEXI_MAIN_FILE);

                    include_once $path . 'templates/build_preview.php';
                    ?>
                </fieldset>
            </td>
        </tr>
    <?php
        return ob_get_clean();
    }

    private function getOptionsConfigFontStyle()
    {
        return array(
            "Normal" => "Normal",
            "Italic" => "Italic",
            "Oblique" => "Oblique",
            "Initial" => "Initial",
            "Inherit" => "Inherit"
        );
    }

    private function getOptionsConfigFontVariant()
    {
        return array(
            "Normal" => "Normal",
            "Small-caps" => "Small-caps",
            "Initial" => "Initial",
            "Inherit" => "Inherit"
        );
    }

    private function getBuildStyle()
    {
        $style = array();

        $fontFamily = $this->settings['font_family'];
        $style['common']['fontFamily'] = $fontFamily;
        $style['correct']['fontFamily'] = $fontFamily;
        $style['error']['fontFamily'] = $fontFamily;

        $fontSize = $this->settings['font_size'];
        $style['common']['fontSize'] = $fontSize;
        $style['correct']['fontSize'] = $fontSize;
        $style['error']['fontSize'] = $fontSize;

        $fontStyle = $this->settings['font_style'];
        $style['common']['fontStyle'] = $fontStyle;
        $style['correct']['fontStyle'] = $fontStyle;
        $style['error']['fontStyle'] = $fontStyle;

        $fontVariant = $this->settings['font_variant'];
        $style['common']['fontVariant'] = $fontVariant;
        $style['correct']['fontVariant'] = $fontVariant;
        $style['error']['fontVariant'] = $fontVariant;

        $letterSpacing = $this->settings['letter_spacing'];
        $style['common']['letterSpacing'] = $letterSpacing;
        $style['correct']['letterSpacing'] = $letterSpacing;
        $style['error']['letterSpacing'] = $letterSpacing;

        $textColorPlaceholder = $this->settings['placeholder_color'];
        $style['common']['::placeholder']['color'] = $textColorPlaceholder;
        $style['correct']['::placeholder']['color'] = $textColorPlaceholder;
        $style['error']['::placeholder']['color'] = $textColorPlaceholder;

        $textColorInput = $this->settings['text_color'];
        $style['common']['color'] = $textColorInput;
        $style['correct']['color'] = $textColorInput;
        $style['error']['color'] = $textColorInput;

        return json_encode($style);
    }

    public function payment_fields()
    {
        $this->tokenization_script();   //hides 'Use new payment card' flag if there aren't saved cards and checks if user is logged in to display the save card option

        $isRecurring = (class_exists("\WC_Subscriptions_Cart") && \WC_Subscriptions_Cart::cart_contains_subscription());

        $payment_payload = \Nexi\WC_Gateway_XPay_API::getInstance()->get_payment_build_payload(\WC_Payment_Gateway::get_order_total());

        if (!$isRecurring) {
            $this->saved_payment_methods();
        }
    ?>
        <fieldset id="wc-<?php echo esc_attr($this->id) ?>-cc-form" class="wc-credit-card-form wc-payment-form">

            <script>
                renderXpayBuild();
            </script>

            <input type="hidden" id="xpay_msg_err" value="<?php echo __("The transaction has been declined, please retry.", 'woocommerce-gateway-nexi-xpay') ?>">

            <input type="hidden" id="xpay_new_payment_info" value="<?php echo htmlentities(json_encode($payment_payload)) ?>">
            <input type="hidden" name="divisa" value="<?php echo htmlentities($payment_payload["divisa"]) ?>">
            <input type="hidden" name="transactionId" id="xpay_build_transactionId" data-new-card-value="<?php echo htmlentities($payment_payload["transactionId"]) ?>">
            <input type="hidden" id="xpay_build_style" value="<?php echo htmlentities($this->getBuildStyle()); ?>">
            <input type="hidden" id="xpay_build_border_color_default" value="<?php echo htmlentities($this->settings['border_color_ok']); ?>">
            <input type="hidden" id="xpay_build_border_color_error" value="<?php echo htmlentities($this->settings['border_color_ko']); ?>">

            <?php
            do_action('woocommerce_credit_card_form_start', $this->id);
            ?>

            <!-- Contiene il form dei dati carta -->
            <div id="xpay-card" style="border: 1px solid  <?php echo $this->settings['border_color_ok']; ?> ; padding: 3px; max-width: 420px;"></div>

            <!-- Contiene gli errori -->
            <div id="xpay-card-errors"></div>
            <br>

            <!-- input valorizzati dopo la chiamata "creaNonce" -->
            <input type="hidden" name="xpayNonce" id="xpayNonce">
            <input type="hidden" name="xpayIdOperazione" id="xpayIdOperazione">
            <input type="hidden" name="xpayTimeStamp" id="xpayTimeStamp">
            <input type="hidden" name="xpayEsito" id="xpayEsito">
            <input type="hidden" name="xpayMac" id="xpayMac">

            <?php
            do_action('woocommerce_credit_card_form_end', $this->id);
            ?>

            <?php
            if ($isRecurring) {
                echo __('Attention, the order for which you are making payment contains recurring payments, payment data will be stored securely by Nexi.', 'woocommerce-gateway-nexi-xpay');
            } else {
            ?>
                <p class="form-row woocommerce-SavedPaymentMethods-saveNew">
                    <input id="save-card" name="save-card" type="checkbox" value="1" style="width:auto;" />
                    <label for="save-card" style="display:inline;"><?php echo __('Remember the payment option.', 'woocommerce-gateway-nexi-xpay'); ?></label>
                </p>
            <?php }
            ?>
            <div class="clear"></div>

        </fieldset>
<?php
    }

    public function process_payment($order_id)
    {
        $order = new \WC_Order($order_id);
        $result = 'failure';

        try {
            $isRecurring = (class_exists("\WC_Subscriptions_Cart") && \WC_Subscriptions_Cart::cart_contains_subscription());
            $isNewCard = false;

            if (isset($_REQUEST["wc-" . $this->id . "-payment-token"])) {
                $isNewCard = $_REQUEST["wc-" . $this->id . "-payment-token"] == "new";
            }

            $save_card = false;
            if (isset($_REQUEST["save-card"])) {
                $save_card = $_REQUEST["save-card"] == "1";
            }

            $nonce = $_REQUEST['xpayNonce'];
            $codTrans = $_REQUEST["transactionId"];
            $divisa = $_REQUEST["divisa"];
            $num_contratto = "";
            $amount = bcmul($order->get_total(), 100);

            if ($save_card || $isRecurring) {
                $user_id = get_current_user_id();

                if ($user_id !== null) {
                    $md5_hash_num_contratto = md5($codTrans . "@" . $user_id . "@" . get_option('nexi_unique'));
                    $num_contratto = "BP" . base_convert($md5_hash_num_contratto, 16, 36);
                }

                \Nexi\WC_Gateway_XPay_API::getInstance()->pagaNonceCreazioneContratto($codTrans, $amount, $nonce, $divisa, $num_contratto);
                WC_Build_Token::save_token($_REQUEST['dettaglioCarta']['brand'], $_REQUEST['dettaglioCarta']['pan'], $_REQUEST['dettaglioCarta']['scadenza'], $num_contratto);

                $order->add_order_note(__("Nexi XPay payment and card tokenization successful", 'woocommerce-gateway-nexi-xpay'));
            } else if (!$isNewCard) {
                \Nexi\WC_Gateway_XPay_API::getInstance()->pagaNonce(false, $codTrans, $amount, $nonce, $divisa);

                $order->add_order_note(__("Nexi XPay payment with tokenized card successful", 'woocommerce-gateway-nexi-xpay'));
            } else {
                \Nexi\WC_Gateway_XPay_API::getInstance()->pagaNonce(true, $codTrans, $amount, $nonce, $divisa);

                $order->add_order_note(__("Nexi XPay payment successful", 'woocommerce-gateway-nexi-xpay'));
            }

            $order->payment_complete();

            WC()->cart->empty_cart();

            WC_Save_Order_Meta::save(
                $order_id,
                \Nexi\WC_Gateway_XPay_API::getInstance()->get_build_alias(),
                $num_contratto,
                $codTrans,
                $_REQUEST['dettaglioCarta']['scadenza']
            );

            $result = 'success';
        } catch (\Throwable $th) {
            Log::actionWarning(__FUNCTION__ . ": error: " . $th->getMessage());
        }

        return array(
            'result' => $result,
            'redirect' => $this->get_return_url($order),
        );
    }

    public function get_saved_payment_method_option_html($token)
    {
        $html = sprintf(
            '<li class="woocommerce-SavedPaymentMethods-token">
                <input id="wc-%1$s-payment-token-%2$s" type="radio" name="wc-%1$s-payment-token" value="%2$s" style="width:auto;" class="woocommerce-SavedPaymentMethods-tokenInput" %4$s />
                <label for="wc-%1$s-payment-token-%2$s">%3$s</label>
            </li>',
            esc_attr($this->id),
            esc_attr($token->get_id()),
            $this->get_token_display_name($token) /* esc_html( $token->get_display_name()) */,
            checked($token->is_default(), true, false)
        );

        return apply_filters('woocommerce_payment_gateway_get_saved_payment_method_option_html', $html, $token, $this);
    }

    private function get_token_display_name($token)
    {
        global $woocommerce;

        // Translators: 1: credit card type 2: last 4 digits 3: expiry month 4: expiry year
        $display = sprintf(
            __('%1$s ending in %2$s (expires %3$s/%4$s)', 'woocommerce-gateway-nexi-xpay'),
            wc_get_credit_card_type_label($token->get_card_type()),
            $token->get_last4(),
            $token->get_expiry_month(),
            substr($token->get_expiry_year(), 2)
        );

        $codTransCvv = substr("BR-" . date('ysdim') . "-" . time(), 0, 30);
        $timestampCvv = time() * 1000;
        $importoCvv = bcmul(floatval(preg_replace('#[^\d.]#', '', $woocommerce->cart->total)), 100, 0);

        $macCvv = WC_Gateway_XPay_API::getInstance()->calculate_mac_for_build_oneclick($codTransCvv, get_woocommerce_currency(), $importoCvv);

        return $display . '
            <div
                class="xpay-card-cvv"
                id="xpay-card-cvv-' . $token->get_token() . '"
                data-wc-id="' . $token->get_id() . '"
                data-token="' . $token->get_token() . '"
                data-codTransCvv="' . $codTransCvv . '"
                data-timestampCvv="' . $timestampCvv . '"
                data-macCvv="' . $macCvv . '"
                style="border: 1px solid ' . $this->settings['border_color_ok'] . '; max-width: 50px;"
            ></div>
            ';
    }
}
