<?php

/**
 * Copyright (c) 2019 Nexi Payments S.p.A.
 *
 * @author      iPlusService S.r.l.
 * @category    Payment Module
 * @package     Nexi XPay
 * @version     6.0.0
 * @copyright   Copyright (c) 2019 Nexi Payments S.p.A. (https://ecommerce.nexi.it)
 * @license     GNU General Public License v3.0
 */

namespace Nexi;

class WC_Gateway_XPay_Register_Available
{

    private static $allowedMethodsByCurrency = array(
        'EUR' => array(
            'PAYPAL',
            'SOFORT',
            'AMAZONPAY',
            'GOOGLEPAY',
            'APPLEPAY',
            'ALIPAY',
            'WECHATPAY',
            'GIROPAY',
            'IDEAL',
            'BCMC',
            'EPS',
            'P24',
            'BANCOMATPAY',
            'SCT',
            // 'MASTERPASS',
            'SKRILL',
            'SKRILL1TAP',
            'MULTIBANCO',
            'MY_BANK',
            'PAGODIL',
        ),
        'CZK' => array(
            'PAYU'
        ),
        'PLN' => array(
            'PAYU',
            'BLIK'
        ),
        'NZD' => array(
            'POLI'
        ),
        'AUD' => array(
            'POLI'
        )
    );
    private static $minAmounts = array(
        'SOFORT' => 10,
        'GIROPAY' => 10,
        'IDEAL' => 10,
        'EPS' => 100,
        'PAYU' => 300,
        'BLIK' => 100,
        'POLI' => 100
    );

    public static function register($paymentGateways)
    {
        $nexiGatewaysHelper = new static();
        return array_merge($paymentGateways, $nexiGatewaysHelper->get_all_nexi_gateways());
    }

    private $paymentGateways;
    private $currency;

    private function get_all_nexi_gateways()
    {
        if (\Nexi\WC_Admin_Page::migrate_data()) {
            return $this->paymentGateways;
        }
        return array();
    }

    private function __construct()
    {
        $this->evaluate_all();
    }

    private function evaluate_all()
    {
        switch (WC_GATEWAY_XPAY_PLUGIN_VARIANT) {
            case 'xpay':
                $mainGateway = new \Nexi\WC_Gateway_XPay_Cards();
                break;
            case 'xpay_build':
                $mainGateway = new \Nexi\WC_Gateway_XPay_Cards_Build();
                break;
            default:
                Log::actionWarning(__('Invalid plugin variant value', 'woocommerce-gateway-nexi-xpay'));
                throw new \Exception(__('Invalid plugin variant value', 'woocommerce-gateway-nexi-xpay'));
        }

        $this->paymentGateways = array();

        // If it's not an admin page (so it is a shop page) and bcmath module isn't installed then the APMs won't be displayed at checkout
        if (!is_admin() && !extension_loaded('bcmath')) {
            return;
        }

        $this->currency = get_woocommerce_currency();

        if (is_admin() || $this->currency == 'EUR') {
            $this->paymentGateways[] = $mainGateway;
        }

        global $pagenow;

        if (!is_admin() || $pagenow !== 'admin.php') {  //$pagenow !== 'admin.php' to hide only on admin pages (wc-settings) otherwise api refund buttons aren't displayed in order page
            $this->evaluate_all_apm();
        }
    }

    private function evaluate_all_apm()
    {
        $jsonAvailableMethods = \WC_Admin_Settings::get_option('xpay_available_methods');
        $availableMethods = json_decode($jsonAvailableMethods, true);
        if (is_array($availableMethods)) {
            foreach ($availableMethods as $am) {
                $this->evaluate_one_apm($am);
            }
        }
    }

    private function evaluate_one_apm($am)
    {
        // The method is an APM with selectedcard support
        if ($am['type'] != 'APM' || $am['selectedcard'] == '') {
            return;
        }

        // If not an admin execute checks
        if (!is_admin()) {
            // The method supports the currency or is in the admin page
            if (!(array_key_exists($this->currency, self::$allowedMethodsByCurrency) && in_array($am['selectedcard'], self::$allowedMethodsByCurrency[$this->currency]))) {
                return;
            }

            // Test for minimum amount. Each APM can have a minimum amount for payment processing
            if (key_exists($am['selectedcard'], self::$minAmounts) && isset(WC()->cart)) {
                $currentCartAmount = bcmul(WC()->cart->total, 100);

                if ($currentCartAmount < self::$minAmounts[$am['selectedcard']]) {
                    return;
                }
            }

            // Test for PagoDIL configuration. Cart must be payable in installable to pay with PagoDIL
            if ($am['selectedcard'] == 'PAGODIL' && isset(WC()->cart)) {
                if (!\Nexi\WC_Pagodil_Widget::isPagodilEnabled()) {
                    return;
                }

                $xpaySettings = \Nexi\WC_Pagodil_Widget::getXPaySettings();

                $pagodilConfig = \Nexi\WC_Pagodil_Widget::getPagodilConfig();

                if (!\Nexi\WC_Pagodil_Widget::isQuoteInstallable($xpaySettings, $pagodilConfig, WC()->cart)) {
                    return;
                }
            }
        }

        $amRecurring = $am['recurring'] == "Y";

        // If all tests are ok then add the APM to the array of gateways
        $this->paymentGateways[] = new \Nexi\WC_Gateway_XPay_APM($am['code'], $am['description'], $am['selectedcard'], $am['image'], $amRecurring);
    }
}
