<?php

/**
 * Plugin Name: Nexi XPay
 * Plugin URI:
 * Description: New Nexi Payments gateway. Official Nexi XPay plugin.
 * Version: 6.0.1
 * Author: Nexi SpA
 * Author URI: https://www.nexi.it
 * Text Domain: woocommerce-gateway-nexi-xpay
 * Domain Path: /lang
 * Copyright: © 2017-2021, Nexi SpA
 * License: GNU General Public License v3.0
 * License URI: http://www.gnu.org/licenses/gpl-3.0.html
 */
if (in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))) {
    if (!defined('ABSPATH')) {
        exit;
    }

    define("WC_GATEWAY_XPAY_VERSION", "6.0.1");

    // The script build-plugin-variants.sh replaces xpay with 'xpay' or 'xpay_build'
    define('WC_GATEWAY_XPAY_PLUGIN_VARIANT', 'xpay');
    define('WC_GATEWAY_XPAY_PLUGIN_COLL', false);

    load_plugin_textdomain('woocommerce-gateway-nexi-xpay', false, dirname(plugin_basename(__FILE__)) . '/lang');

    include_once realpath(plugin_dir_path(__FILE__)) . DIRECTORY_SEPARATOR . 'src' . DIRECTORY_SEPARATOR . "autoload.php";

    add_filter('woocommerce_payment_gateways', "\Nexi\WC_Gateway_XPay_Register_Available::register");

    // Register endpoint in the rest api for s2s notification API, for post payment redirect url and for cancel url
    add_action('rest_api_init', '\Nexi\WC_Gateway_XPay_Process_Completion::rest_api_init');

    \Nexi\WC_Gateway_XPay_Process_Completion::register();

    \Nexi\WC_Pagodil_Widget::register();

    define('WC_ECOMMERCE_GATEWAY_NEXI_MAIN_FILE', __FILE__);

    function xpay_gw_wp_enqueue_scripts()
    {
        wp_enqueue_script('xpay-checkout', plugins_url('assets/js/xpay.js', __FILE__), array('jquery'));
        wp_enqueue_style('xpay-checkout', plugins_url('assets/css/xpay.css', __FILE__));

        wp_enqueue_script('xpay-build-checkout', plugins_url('assets/js/xpay-build.js', __FILE__), array('jquery'));
    }

    add_action('admin_init', '\Nexi\WC_Admin_Page::init');

    add_action('wp_enqueue_scripts', 'xpay_gw_wp_enqueue_scripts');

    if (!defined("WC_WOOCOMMERCE_GATEWAY_NEXI_XPAY_WOOCOMMERCE_VERSION")) {
        if (!function_exists('get_plugins')) {
            require_once ABSPATH . 'wp-admin/includes/plugin.php';
        }

        $plugins = get_plugins();

        define("WC_WOOCOMMERCE_GATEWAY_NEXI_XPAY_WOOCOMMERCE_VERSION", $plugins["woocommerce/woocommerce.php"]["Version"]);
    }

    if (!defined("WC_WOOCOMMERCE_GATEWAY_NEXI_XPAY_DEFAULT_LOGO_URL")) {
        $default_logo_url = plugins_url('assets/images/logo.jpg', plugin_basename(__FILE__));

        define("WC_WOOCOMMERCE_GATEWAY_NEXI_XPAY_DEFAULT_LOGO_URL", $default_logo_url);
    }

    function xpay_plugin_activation()
    {
        $nexi_unique = get_option("nexi_unique");

        if ($nexi_unique == "") {
            update_option('nexi_unique', uniqid());
        }
    }

    register_activation_hook(__FILE__, 'xpay_plugin_activation');

    function xpay_plugin_action_links($links)
    {
        $plugin_links = array(
            '<a href="' . esc_url(admin_url('admin.php?page=wc-settings&tab=checkout&section=' . WC_GATEWAY_XPAY_PLUGIN_VARIANT)) . '">' . __('Settings') . '</a>',
        );

        return array_merge($plugin_links, $links);
    }

    add_filter('plugin_action_links_' . plugin_basename(__FILE__), 'xpay_plugin_action_links');

    add_action('init', '\Nexi\WC_Gateway_XPay_Cards::addNexiPendingPaymentOrderStatus');

    add_filter('wc_order_statuses', '\Nexi\WC_Gateway_XPay_Cards::wcOrderStatusesFilter');

    add_filter('woocommerce_valid_order_statuses_for_payment_complete', '\Nexi\WC_Gateway_XPay_Cards::validOrderStatusesForPaymentCompleteFilter');
}

/**
 * checks if the requested information is available as per new configuration format and returns it
 * if it is not available in the new format it is searched in the old one
 *
 * @param int $order_id
 * @param string $key   'alias' | 'scadenza_pan' | 'num_contratto' | 'codTrans'
 * @return mixed
 */
function get_xpay_post_meta($order_id, $key)
{
    $ret = get_post_meta($order_id, '_xpay_' . $key, true);

    if ($ret != "") {
        return $ret;
    }

    $xpay_details_order = get_post_meta($order_id, "xpay_details_order", true);

    if ($xpay_details_order != "") {
        $details_order = json_decode($xpay_details_order);

        update_post_meta($order_id, "_xpay_" . "alias", $details_order->alias);
        update_post_meta($order_id, "_xpay_" . "scadenza_pan", $details_order->scadenza_pan);
        update_post_meta($order_id, "_xpay_" . "num_contratto", $details_order->num_contratto);
        update_post_meta($order_id, "_xpay_" . "codTrans", $details_order->codTrans);

        return $details_order->{$key};
    }

    return "";
}


/**
 * checks if bcmath is available and returns the divison result calculated with its function, otherwise returns the result calculated with basic arithmetic operators
 *
 * @param int|float|string $dividend
 * @param int|float|string $divisor
 * @param integer $decimals
 * @return float
 */
function div_bcdiv($dividend, $divisor, $decimals = 2)
{
    if (extension_loaded('bcmath')) {
        return (float) bcdiv((string) $dividend, (string) $divisor, $decimals);
    }

    return round($dividend / $divisor, $decimals);
}
